#!/bin/bash
# CPL - Contest test script

LLVM_CPL_PASS=libLLVMCpl-Contest.so

# Directory containing the tests
TESTDIR="$1"

function run_opt()
{
	test_file=$1
	optimized_file=$2
	cat $test_file | $OPT -load $passesFile $opt_args > $optimized_file
}

function run_test()
{
	test_bc=$1
	optimized_file_bc=$2

	# Run the original bc file
	reference_result=$($LLI $test_bc)
	reference_return_value=$?

	# Run opt with our optimization passes
	run_opt $test_bc $optimized_file_bc
	new_result=$($LLI $optimized_file_bc)
	new_return_value=$?

	if [ "$reference_result" == "$new_result" ] && [ "$reference_return_value" == "$new_return_value" ]; then
		echo 1
	else
		echo 0
	fi
}

function get_text_size()
{
	object_file=$1.o
	$LLC $1 -march=arm -filetype=obj -o $object_file 2>/dev/null

	int_size=$($LLSIZE -A $object_file | grep -oE '^.text +([0-9]+)' | grep -oE '[0-9]+$')
	echo $int_size
}

function get_Oz_text_size()
{
	out_opt_Os=$1.os
	cat $1 | $OPT -S -Oz > $out_opt_Os
	size=$(get_text_size $out_opt_Os)
	#rm $out_opt_Os
	echo $size
}

# Detect llvm location and pass names

LLVM_BIN_DIR=$(llvm-config --bindir)
OPT=$LLVM_BIN_DIR/opt
LLI=$LLVM_BIN_DIR/lli
LLC=$LLVM_BIN_DIR/llc
LLSIZE=$LLVM_BIN_DIR/llvm-size

passesFile=$(find . -name $LLVM_CPL_PASS -print -quit)
if [ ! -f "$passesFile" ]; then echo "$LLVM_CPL_PASS not found"; exit; fi

opt_args=$(echo $($OPT -load $passesFile --help | grep -oE -- '-cpl-[^ ]+'))


# main loop for tests
final_score=0
printf "%-32s %-10s %-10s %s\n" "Test name" "Status" "Score" "Default"
	tests=$(find ${TESTDIR:-cplmark} -name '*.ll' | sort -V)
	for test_ll in $tests; do
		optimized_file_bc=$test_ll".opt"

		# Run the test
		result=$(run_test $test_ll $optimized_file_bc)
		test_file=$(basename $test_ll .ll)

		# Pretty print
		if [ "$result" == "1" ]; then
			default_score=$(get_text_size $test_ll)
			score=$(get_text_size $optimized_file_bc)
			#Oz_score=$(get_Oz_text_size $test_ll)
			final_score=$(($final_score + $score))
			printf "%s %-30s %-10s %-10s %s\n" '-' "$test_file" "PASSED" "$score" "$default_score"
		else
			printf "%s %-30s %-1s\n" '-' "$test_file" "FAILED"
		fi

		# Remove this if you want to keep the optimized bc file 
		rm $optimized_file_bc
	done
echo "=============================================================================="
echo "Final score =$final_score="
    
